@@ -52,6 +52,9 @@ public final class ExtendRule implements RuleInt {
     private ExtendRuleConfig extensionConfig;
     private Float confidenceOfSeedRule;
     protected Rule rule;
+    //this field contains rule quality values obtained after rules above the current
+    //rule were removed during pruning
+    private RuleQuality ruleQualityInRuleList;
     public float getConfidenceOfSeedRule()
     {
             return confidenceOfSeedRule;
@@ -798,4 +801,13 @@ public ExtendRule removeRedundantAttributes(){
         return rule.getArulesRepresentation();
     }
 
+    void setQualityInRuleList(RuleQuality ruleQuality) {
+        ruleQualityInRuleList = ruleQuality;
+        }
+
+    
+    public RuleQuality getQualityInRuleList() {
+        return(ruleQualityInRuleList);
+    }
+
 }
